SET DEFINE OFF;


SET VERIFY OFF;


CREATE OR REPLACE TYPE v_type AS OBJECT (v_num NUMBER);

/

CREATE OR REPLACE TYPE V_TABLE_TYPE AS TABLE OF v_type

/

CREATE OR REPLACE PACKAGE export_excel_pkg
AS
   PROCEDURE get_usable_sql (p_sql_in IN VARCHAR2, p_sql_out OUT VARCHAR2);

   PROCEDURE print_header (no_of_cols IN NUMBER);

   PROCEDURE print_footer (no_of_cols IN NUMBER);

   PROCEDURE print_report_header (
      p_region    IN       VARCHAR2,
      p_page_id   IN       NUMBER,
      p_app_id    IN       NUMBER,
      p_error     OUT      VARCHAR2
   );

   PROCEDURE print_report_values (
      p_page_id      IN       NUMBER,
      p_app_id       IN       NUMBER,
      p_app_user     IN       VARCHAR2,
      p_session_id   IN       VARCHAR2,
      p_error        OUT      VARCHAR2
   );

   PROCEDURE print_report (
      p_region      IN   VARCHAR2,
      p_file_name   IN   VARCHAR2 DEFAULT 'excel_report',
      p_page_id     IN   NUMBER DEFAULT v ('APP_PAGE_ID')
   );
END export_excel_pkg;
/


CREATE OR REPLACE PACKAGE BODY export_excel_pkg
AS
------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------
   v_top_number_of_columns   NUMBER;
--   v_top_column_list         DBMS_SQL.desc_tab;
   v_top_column_list         apex_application_global.vc_arr2;
   v_top_region_sql          VARCHAR2 (32767);
   v_loop_error              VARCHAR2 (32767);

------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------
   PROCEDURE get_usable_sql (p_sql_in IN VARCHAR2, p_sql_out OUT VARCHAR2)
   IS
      v_sql      VARCHAR2 (32767);
      v_names    DBMS_SQL.varchar2_table;
      v_pos      NUMBER;
      v_length   NUMBER;
      v_exit     NUMBER;
   BEGIN
      v_sql := p_sql_in;
      v_names := wwv_flow_utilities.get_binds (v_sql);

      FOR i IN 1 .. v_names.COUNT
      LOOP

         <<do_it_again>>
         v_pos := INSTR (LOWER (v_sql), LOWER (v_names (i)));
         v_length := LENGTH (LOWER (v_names (i)));
         v_sql :=
               SUBSTR (v_sql, 1, v_pos - 1)
            || v_names (i)
            || SUBSTR (v_sql, v_pos + v_length);
         v_sql :=
            REPLACE (v_sql,
                     UPPER (v_names (i)),
                        '(SELECT v('''
                     || LTRIM (v_names (i), ':')
                     || ''') FROM DUAL)'
                    );

         IF INSTR (LOWER (v_sql), LOWER (v_names (i))) > 0
         THEN
            GOTO do_it_again;
         END IF;
      END LOOP;

      p_sql_out := v_sql;
   END get_usable_sql;

------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------
   PROCEDURE print_header (no_of_cols IN NUMBER)
   AS
      v_head   VARCHAR2 (32767);
   BEGIN
      v_head :=
            '<html xmlns:o="urn:schemas-microsoft-com:office:office"'
         || 'xmlns:x="urn:schemas-microsoft-com:office:excel"'
         || 'xmlns="http://www.w3.org/TR/REC-html40">'
         || '<head>'
         || '<meta http-equiv=Content-Type content="text/html; charset=UTF-8">'
         || '<meta name=ProgId content=Excel.Sheet>'
         || '<meta name=Generator content="Microsoft Excel 9">'
         || '<link rel=File-List href="./Sheet1-File/filelist.xml">'
         || '<link rel=Edit-Time-Data href="./Sheet1-File/editdata.mso">'
         || '<link rel=OLE-Object-Data href="./Sheet1-File/oledata.mso">'
         || '<!--[if gte mso 9]><xml>'
         || '<o:DocumentProperties>'
         || '<o:Author></o:Author>'
         || '<o:LastAuthor></o:LastAuthor>'
         || '<o:Created></o:Created>'
         || '<o:LastSaved></o:LastSaved>'
         || '<o:Company></o:Company>'
         || '<o:Version>9.3821</o:Version>'
         || '</o:DocumentProperties>'
         || '<o:OfficeDocumentSettings>'
         || '<o:DownloadComponents/>'
         || '<o:LocationOfComponents HRef="file:"/>'
         || '</o:OfficeDocumentSettings>'
         || '</xml><![endif]-->'
         || '<style>'
         || '<!--table'
         || '{mso-displayed-decimal-separator:"\,";'
         || 'mso-displayed-thousand-separator:"\.";}'
         || '@page'
         || '{margin:.98in .79in .98in .79in;'
         || 'mso-header-margin:.49in;'
         || 'mso-footer-margin:.49in;}'
         || 'tr'
         || '{mso-height-source:auto;}'
         || 'col'
         || '{mso-width-source:auto;}'
         || 'br'
         || '{mso-data-placement:same-cell;}'
         || '.style0'
         || '{mso-number-format:General;'
         || 'text-align:general;'
         || 'vertical-align:bottom;'
         || 'white-space:nowrap;'
         || 'mso-rotate:0;'
         || 'mso-background-source:auto;'
         || 'mso-pattern:auto;'
         || 'color:windowtext;'
         || 'font-size:10.0pt;'
         || 'font-weight:400;'
         || 'font-style:normal;'
         || 'text-decoration:none;'
         || 'font-family:Arial;'
         || 'mso-generic-font-family:auto;'
         || 'mso-font-charset:0;'
         || 'border:none;'
         || 'mso-protection:locked visible;'
         || 'mso-style-name:Standard;'
         || 'mso-style-id:0;}'
         || 'td'
         || '{mso-style-parent:style0;'
         || 'padding-top:1px;'
         || 'padding-right:1px;'
         || 'padding-left:1px;'
         || 'mso-ignore:padding;'
         || 'color:windowtext;'
         || 'font-size:10.0pt;'
         || 'font-weight:400;'
         || 'font-style:normal;'
         || 'text-decoration:none;'
         || 'font-family:Arial;'
         || 'mso-generic-font-family:auto;'
         || 'mso-font-charset:0;'
-- bkahles, 14.06.2007
--         || 'mso-number-format:General;'
         || 'mso-number-format:\@;'
         || 'text-align:general;'
         || 'vertical-align:bottom;'
         || 'border:none;'
         || 'mso-background-source:auto;'
         || 'mso-pattern:auto;'
         || 'mso-protection:locked visible;'
         || 'white-space:nowrap;'
         || 'mso-rotate:0;}'
         || '.xl24'
         || '{mso-style-parent:style0;'
         || 'border:.5pt solid windowtext;'
         || 'background:#FFEFD5;'
         || 'mso-pattern:auto none;}'
         || '.xl25'
         || '{mso-style-parent:style0;'
         || 'border:.5pt solid windowtext;'
         || 'background:#D9E8F2;'
         || 'mso-pattern:auto none;}'
         || '-->'
         || '</style>'
         || '<!--[if gte mso 9]><xml>'
         || '<x:ExcelWorkbook>'
         || '<x:ExcelWorksheets>'
         || '<x:ExcelWorksheet>'
         || '<x:Name>Tabelle1</x:Name>'
         || '<x:WorksheetOptions>'
         || '<x:DefaultRowHeight>264</x:DefaultRowHeight>'
         || '<x:DefaultColWidth>10</x:DefaultColWidth>'
         || '<x:Selected/>'
         || '<x:ProtectContents>False</x:ProtectContents>'
         || '<x:ProtectObjects>False</x:ProtectObjects>'
         || '<x:ProtectScenarios>False</x:ProtectScenarios>'
         || '</x:WorksheetOptions>'
         || '</x:ExcelWorksheet>'
         || '<x:ExcelWorksheet>'
         || '<x:Name>Tabelle2</x:Name>'
         || '<x:WorksheetOptions>'
         || '<x:DefaultRowHeight>264</x:DefaultRowHeight>'
         || '<x:DefaultColWidth>10</x:DefaultColWidth>'
         || '<x:ProtectContents>False</x:ProtectContents>'
         || '<x:ProtectObjects>False</x:ProtectObjects>'
         || '<x:ProtectScenarios>False</x:ProtectScenarios>'
         || '</x:WorksheetOptions>'
         || '</x:ExcelWorksheet>'
         || '<x:ExcelWorksheet>'
         || '<x:Name>Tabelle3</x:Name>'
         || '<x:WorksheetOptions>'
         || '<x:DefaultRowHeight>264</x:DefaultRowHeight>'
         || '<x:DefaultColWidth>10</x:DefaultColWidth>'
         || '<x:ProtectContents>False</x:ProtectContents>'
         || '<x:ProtectObjects>False</x:ProtectObjects>'
         || '<x:ProtectScenarios>False</x:ProtectScenarios>'
         || '</x:WorksheetOptions>'
         || '</x:ExcelWorksheet>'
         || '</x:ExcelWorksheets>'
         || '<x:WindowHeight>10620</x:WindowHeight>'
         || '<x:WindowWidth>18240</x:WindowWidth>'
         || '<x:WindowTopX>480</x:WindowTopX>'
         || '<x:WindowTopY>84</x:WindowTopY>'
         || '<x:ProtectStructure>False</x:ProtectStructure>'
         || '<x:ProtectWindows>False</x:ProtectWindows>'
         || '</x:ExcelWorkbook>'
         || '</xml><![endif]-->'
         || '</head>';
      HTP.prn (v_head);
   END print_header;

------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------
   PROCEDURE print_footer (no_of_cols IN NUMBER)
   AS
      v_footer   VARCHAR2 (32767);
   BEGIN
      v_footer :=
         q'!
    <![if supportMisalignedColumns]>
 <tr>
 </tr>

 <![endif]>
 </table>

 </body>

 </html>!';
      HTP.prn (v_footer);
   END print_footer;

------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------
   PROCEDURE print_report_header (
      p_region    IN       VARCHAR2,
      p_page_id   IN       NUMBER,
      p_app_id    IN       NUMBER,
      p_error     OUT      VARCHAR2
   )
   AS
      v_header               VARCHAR2 (32767);
      v_region_sql           VARCHAR2 (32767);
      v_column_cursor        PLS_INTEGER;
      v_column_list          DBMS_SQL.desc_tab;
      v_number_of_cols       NUMBER;
      v_column_header_list   VARCHAR2 (32767);
      v_column_alias_list    VARCHAR2 (32767);
      v_column_select_list   VARCHAR2 (32767);
      v_header_arr           apex_application_global.vc_arr2;
   BEGIN
      SELECT region_source
        INTO v_region_sql
        FROM apex_application_page_regions
       WHERE region_id = TO_NUMBER (LTRIM (p_region, 'R'))
         AND page_id = p_page_id
         AND application_id = p_app_id;

      --     v_column_cursor := DBMS_SQL.open_cursor;
      --     DBMS_SQL.parse (v_column_cursor, v_region_sql, DBMS_SQL.native);
      --     DBMS_SQL.describe_columns (v_column_cursor,
      --                                v_number_of_cols,
      --                                v_column_list
      --                               );
      --     DBMS_SQL.close_cursor (v_column_cursor);
      v_header := q'!
      <body link=blue vlink=purple>

<table>
 <tr>!';
      v_header := v_header;
      --     FOR i IN 1 .. v_number_of_cols
      --     LOOP
      --        v_header :=
      --              v_header
      --           || '<td class=xl24>'
      --           || v_column_list (i).col_name
      --           || '</td>';
      --     END LOOP;
           --
           -- Export only the columns with checked "Include in Export" flag under "Print Attributes"
           -- and take into account the column display order and any number/date column formatting
           -- In first row export column headings instead of column aliases.
      v_number_of_cols := 0;
      v_column_header_list := '';
      v_column_alias_list := '';
      v_column_select_list := '';

      FOR c IN (SELECT   column_alias, NVL (heading, column_alias) heading,
                         format_mask
                    FROM apex_application_page_rpt_cols
                   WHERE page_id = p_page_id
                     AND application_id = p_app_id
                     AND region_id = TO_NUMBER (LTRIM (p_region, 'R'))
                     AND include_in_export = 'Yes'
                -- and column_is_hidden = 'No'
                ORDER BY display_sequence)
      LOOP
         v_number_of_cols := v_number_of_cols + 1;
         v_column_header_list :=
                 v_column_header_list || ';' || REPLACE (c.heading, ';', ' ');
         v_column_alias_list := v_column_alias_list || ';' || c.column_alias;

         -- apply column formatting
         IF c.format_mask IS NOT NULL
         THEN
            v_column_select_list :=
                  v_column_select_list
               || ',to_char('
               || c.column_alias
               || ','''
               || c.format_mask
               || ''') '
               || c.column_alias;
         ELSE
            v_column_select_list :=
                                v_column_select_list || ',' || c.column_alias;
         END IF;
      END LOOP;

      v_column_header_list := SUBSTR (v_column_header_list, 2);
      v_header_arr := apex_util.string_to_table (v_column_header_list, ';');
      v_column_alias_list := SUBSTR (v_column_alias_list, 2);
      v_top_column_list :=
                          apex_util.string_to_table (v_column_alias_list, ';');
      v_column_select_list :=
                     'select ' || SUBSTR (v_column_select_list, 2)
                     || ' from (';

      FOR i IN 1 .. v_header_arr.COUNT
      LOOP
         v_header :=
                  v_header || '<td class=xl24>' || v_header_arr (i)
                  || '</td>';
      END LOOP;

      v_header := v_header || '</tr>';
      HTP.prn (v_header);
      export_excel_pkg.v_top_number_of_columns := v_number_of_cols;
--      export_excel_pkg.v_top_column_list := v_column_list;
      export_excel_pkg.get_usable_sql (v_region_sql,
                                       export_excel_pkg.v_top_region_sql
                                      );
      v_top_region_sql := v_column_select_list || v_top_region_sql || ')';
   EXCEPTION
      WHEN OTHERS
      THEN
         p_error := 'Report Header Error: ' || SQLERRM || ' / ' || SQLCODE;
   END print_report_header;

------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------
   PROCEDURE print_report_values (
      p_page_id      IN       NUMBER,
      p_app_id       IN       NUMBER,
      p_app_user     IN       VARCHAR2,
      p_session_id   IN       VARCHAR2,
      p_error        OUT      VARCHAR2
   )
   AS
      v_values             VARCHAR2 (32767);
      v_column_list_name   VARCHAR2 (32767);
      v_loop_exception     EXCEPTION;
      v_sql                VARCHAR2 (32767);
   BEGIN
      v_column_list_name := 'v_print_row := ''<tr>'';';

      FOR i IN 1 .. v_top_number_of_columns
      LOOP
         v_column_list_name :=
               v_column_list_name
            || 'v_print_row := v_print_row || ''<td class=xl25>''||'
            || 'c.'
--            || v_top_column_list (i).col_name
            || v_top_column_list (i)
            || '||  ''</td>''||chr(10);';
      END LOOP;

      v_column_list_name :=
                v_column_list_name || 'v_print_row := v_print_row||''</tr>'';';

      BEGIN
         v_sql :=
               'DECLARE v_print_row varchar2(32767); '
            || 'BEGIN '
            || 'HTMLDB_CUSTOM_AUTH.define_user_session ('
            || ''''
            || p_app_user
            || ''''
            || ', '
            || p_session_id
            || '); '
            || 'HTMLDB_APPLICATION.g_flow_id := '
            || p_app_id
            || '; '
            || 'FOR c IN ('
            || v_top_region_sql
            || ') LOOP '
            || 'v_print_row := ''<tr>''; '
            || v_column_list_name
            || ' HTP.prn (v_print_row);'
            || ' END LOOP;'
            || 'EXCEPTION '
            || 'WHEN OTHERS THEN '
            || 'NULL;'
            || 'END;';

         EXECUTE IMMEDIATE v_sql;
      EXCEPTION
         WHEN v_loop_exception
         THEN
            p_error := export_excel_pkg.v_loop_error;
         WHEN OTHERS
         THEN
            p_error :=
                  'Report Values Error: '
               || SQLERRM
               || ' / '
               || SQLCODE
               || ' / '
               || v_top_region_sql;
      END;
   END print_report_values;

------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------
   PROCEDURE print_report (
      p_region      IN   VARCHAR2,
      p_file_name   IN   VARCHAR2 DEFAULT 'excel_report',
      p_page_id     IN   NUMBER DEFAULT v ('APP_PAGE_ID')
   )
   AS
      v_app_id           NUMBER           := v ('APP_ID');
      v_app_user         VARCHAR2 (200)   := v ('APP_USER');
      v_session_id       VARCHAR2 (50)    := v ('APP_SESSION');
      v_err              VARCHAR2 (32767);
      v_rep_header_err   VARCHAR2 (32767);
      v_rep_values_err   VARCHAR2 (32767);
      v_file_name        VARCHAR2 (300);
      v_exception        EXCEPTION;
   BEGIN
      EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_LANGUAGE = GERMAN';

      v_file_name := p_file_name || '.xls';
      -- Set the MIME type
      OWA_UTIL.mime_header ('application/octet', FALSE);
-- Set the name of the file
      HTP.p ('Content-Disposition: attachment; filename="' || v_file_name
             || '"'
            );
-- Close the HTTP Header
      OWA_UTIL.http_header_close;
      export_excel_pkg.print_header (1);
      export_excel_pkg.print_report_header (p_region,
                                            p_page_id,
                                            v_app_id,
                                            v_rep_header_err
                                           );

      IF v_rep_header_err IS NOT NULL
      THEN
         RAISE v_exception;
      END IF;

      export_excel_pkg.print_report_values (p_page_id,
                                            v_app_id,
                                            v_app_user,
                                            v_session_id,
                                            v_rep_values_err
                                           );

      IF v_rep_values_err IS NOT NULL
      THEN
         RAISE v_exception;
      END IF;

      export_excel_pkg.print_footer (1);
-- Send an error code so that the
-- rest of the HTML does not render
      HTMLDB_APPLICATION.g_unrecoverable_error := TRUE;
   EXCEPTION
      WHEN v_exception
      THEN
         HTP.prn (v_rep_header_err);
         HTP.prn (v_rep_values_err);
         HTMLDB_APPLICATION.g_unrecoverable_error := TRUE;
      WHEN OTHERS
      THEN
         v_err :=
               'Printing Report Error:'
            || SQLERRM
            || ' / '
            || SQLCODE
            || ' / '
            || 'Report Header Error:'
            || v_rep_header_err
            || ' / '
            || 'Report Values Error: '
            || v_rep_values_err;
         HTP.prn (v_err);
         HTMLDB_APPLICATION.g_unrecoverable_error := TRUE;
   END print_report;
------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------
END export_excel_pkg;
